import { FC } from 'react'
import styled from 'styled-components'
import * as React from 'react'

export interface ToggleProps {
  name: string
  onBlur: <T>(event?: React.FocusEvent<T>) => void
  onChange: <T>(event: React.ChangeEvent<T> | any) => void
  onFocus: <T>(event?: React.FocusEvent<T>) => void
  value: any
  input: any
  checked?: boolean
  disabled?: boolean
}

export const Toggle: FC<ToggleProps> = props => {
  return (
    <ToggleElement>
      <ToggleInput id={props.name} type="checkbox" {...props.input} />
      <ToggleLabel htmlFor={props.name} role="switch" disabled={props.disabled}>
        <ToggleSwitch checked={props.input.value ? true : false}>
          <span></span>
        </ToggleSwitch>
      </ToggleLabel>
    </ToggleElement>
  )
}

const ToggleElement = styled.div`
  display: inline-block;
  position: relative;
  width: 40px;
  height: 20px;
  margin: 0;
`

const ToggleLabel = styled.label<{ disabled?: boolean }>`
  background: none;
  color: inherit;
  padding: 0;
  opacity: ${props => (props.disabled ? '0.4' : '1')};
  outline: none;
  width: 3rem;
  height: 1.75rem;
  pointer-events: ${props => (props.disabled ? 'none' : 'inherit')};
`

const ToggleSwitch = styled.div<{ checked: boolean }>`
  position: relative;
  width: 3rem;
  height: 1.75rem;
  border-radius: 1.5rem;
  background-color: white;
  border: 1px solid #edecf3;
  pointer-events: none;
  margin-left: -2px;
  span {
    position: absolute;
    border-radius: 1.5rem;
    left: 2px;
    top: 50%;
    width: calc(1.75rem - 6px);
    height: calc(1.75rem - 6px);
    background: ${p => (p.checked ? '#0084FF' : '#E1DDEC')};
    transform: translate3d(${p => (p.checked ? '1.25rem' : '0')}, -50%, 0);
    transition: all 150ms ease-out;
  }
`

const ToggleInput = styled.input`
  position: absolute;
  left: 0;
  top: 0;
  width: 3rem;
  height: 1.75rem;
  opacity: 0;
  margin: 0;
  cursor: ${props => (props.disabled ? 'not-allowed' : 'pointer')};
`
